#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphFactory
from pyams_content.component.paragraph.interfaces.header import IHeaderParagraph
from pyams_i18n.interfaces import II18n

# import packages
from pyams_content.component.paragraph import BaseParagraph
from pyams_utils.registry import utility_config
from pyams_utils.text import get_text_start
from zope.interface import implementer
from zope.schema.fieldproperty import FieldProperty

from pyams_content import _


@implementer(IHeaderParagraph)
class HeaderParagraph(BaseParagraph):
    """Header paragraph"""

    icon_class = 'fa-header'
    icon_hint = _("Header")

    @property
    def title(self):
        header = II18n(self).query_attribute('header')
        return get_text_start(header, 50, 10)

    header = FieldProperty(IHeaderParagraph['header'])


@utility_config(name='Header', provides=IParagraphFactory)
class HTMLParagraphFactory(object):
    """HTML paragraph factory"""

    name = _("Header paragraph")
    content_type = HeaderParagraph
