#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IBaseParagraph, IParagraphFactorySettings
from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_form.interfaces.form import IFormHelp
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces.menu import IPropertiesMenu
from pyams_zmi.layer import IAdminLayer

# import packages
from pyams_form.form import AJAXEditForm, AJAXAddForm
from pyams_form.help import FormHelp
from pyams_form.security import ProtectedFormObjectMixin
from pyams_pagelet.pagelet import pagelet_config
from pyams_skin.viewlet.menu import MenuItem
from pyams_skin.viewlet.toolbar import ToolbarMenuItem
from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyramid.view import view_config
from z3c.form import field

from pyams_content import _


#
# Default paragraphs settings
#

@viewlet_config(name='default-paragraphs.menu', context=IParagraphFactorySettings, layer=IAdminLayer,
                manager=IPropertiesMenu, permission=MANAGE_TOOL_PERMISSION, weight=5)
class DefaultParagraphsSettingsMenu(MenuItem):
    """Default paragraphs settings menu"""

    label = _("Paragraphs types...")
    icon_class = 'fa-paragraph'
    url = 'default-paragraphs.html'
    modal_target = True


@pagelet_config(name='default-paragraphs.html', context=IParagraphFactorySettings, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class DefaultParagraphsEditForm(AdminDialogEditForm):
    """Default paragraphs edit form"""

    legend = _("Paragraphs types")

    fields = field.Fields(IParagraphFactorySettings)
    ajax_handler = 'default-paragraphs.json'
    edit_permission = MANAGE_TOOL_PERMISSION


@view_config(name='default-paragraphs.json', context=IParagraphFactorySettings, request_type=IPyAMSLayer,
             permission=MANAGE_TOOL_PERMISSION, renderer='json', xhr=True)
class DefaultParagraphAJAXEditForm(AJAXEditForm, DefaultParagraphsEditForm):
    """Default paragraphs edit form, JSON renderer"""


@adapter_config(context=(IParagraphFactorySettings, IPyAMSLayer, DefaultParagraphsEditForm), provides=IFormHelp)
class DefaultParagraphsEditFormHelp(FormHelp):
    """Default paragraphs edit form help"""

    message = _("You can define which types of paragraphs are allowed in this container.\n\n"
                "Default paragraphs will be added automatically (in selected order) to any new created content.\n\n"
                "NOTICE: removing types from allowed types list will have no effect on already created contents!")
    message_format = 'rest'


#
# Base paragraph forms
#

class BaseParagraphAddMenu(ProtectedFormObjectMixin, ToolbarMenuItem):
    """Base paragraph add menu"""

    paragraph_type = None
    modal_target = True

    def __new__(cls, context, request, view, manager):
        settings = get_parent(context, IParagraphFactorySettings)
        if (settings is not None) and (cls.paragraph_type not in settings.allowed_paragraphs):
            return None
        return ToolbarMenuItem.__new__(cls)


class BaseParagraphAJAXAddForm(AJAXAddForm):
    """Base paragraph AJAX add form"""

    def get_ajax_output(self, changes):
        return {'status': 'reload',
                'location': '#paragraphs.html'}


class BaseParagraphAJAXEditForm(AJAXEditForm):
    """Base paragraph AJAX edit form"""

    def get_ajax_output(self, changes):
        output = super(BaseParagraphAJAXEditForm, self).get_ajax_output(changes)
        if 'title' in changes.get(IBaseParagraph, ()):
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.paragraphs.refreshParagraph',
                            'object_name': self.context.__name__,
                            'title': II18n(self.context).query_attribute('title', request=self.request),
                            'visible': self.context.visible}
            })
        return output
