#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.association.interfaces import IAssociationTarget
from pyams_content.component.association.zmi.interfaces import IAssociationsParentForm
from pyams_content.component.illustration.interfaces import IIllustration, IIllustrationRenderer
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphSummary
from pyams_content.component.paragraph.interfaces.html import IHTMLParagraph
from pyams_content.component.paragraph.zmi.interfaces import IParagraphInnerEditor
from pyams_content.interfaces import MANAGE_CONTENT_PERMISSION
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IInnerForm, IEditFormButtons
from pyams_i18n.interfaces import II18n
from pyams_skin.interfaces.viewlet import IToolbarAddingMenu
from pyams_skin.layer import IPyAMSLayer
from pyams_zmi.interfaces import IPropertiesEditForm
from z3c.form.interfaces import INPUT_MODE

# import packages
from pyams_content.component.association.zmi import AssociationsTable
from pyams_content.component.paragraph.html import HTMLParagraph
from pyams_content.component.paragraph.zmi import BaseParagraphAJAXAddForm, BaseParagraphAJAXEditForm, \
    BaseParagraphAddMenu
from pyams_content.component.paragraph.zmi.container import ParagraphContainerView, ParagraphContainerTable, \
    ParagraphTitleToolbarViewletManager
from pyams_pagelet.pagelet import pagelet_config
from pyams_template.template import template_config

from pyams_utils.adapter import adapter_config
from pyams_utils.traversing import get_parent
from pyams_viewlet.viewlet import viewlet_config, BaseContentProvider
from pyams_zmi.form import AdminDialogAddForm, AdminDialogEditForm
from pyramid.threadlocal import get_current_registry
from pyramid.view import view_config
from z3c.form import field, button
from zope.interface import implementer, Interface

from pyams_content import _


#
# HTML paragraph
#

@viewlet_config(name='add-html-paragraph.menu', context=IParagraphContainerTarget, view=ParagraphContainerView,
                layer=IPyAMSLayer, manager=IToolbarAddingMenu, weight=50)
class HTMLParagraphAddMenu(BaseParagraphAddMenu):
    """HTML paragraph add menu"""

    label = _("Add HTML paragraph...")
    label_css_class = 'fa fa-fw fa-html5'
    url = 'add-html-paragraph.html'
    paragraph_type = 'HTML'


@pagelet_config(name='add-html-paragraph.html', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HTMLParagraphAddForm(AdminDialogAddForm):
    """HTML paragraph add form"""

    legend = _("Add new HTML paragraph")
    dialog_class = 'modal-max'
    icon_css_class = 'fa fa-fw fa-html5'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IHTMLParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'add-html-paragraph.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HTMLParagraphAddForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            self.widgets['body'].widget_css_class = 'textarea'

    def create(self, data):
        return HTMLParagraph()

    def add(self, object):
        IParagraphContainer(self.context).append(object)


@view_config(name='add-html-paragraph.json', context=IParagraphContainerTarget, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphAJAXAddForm(BaseParagraphAJAXAddForm, HTMLParagraphAddForm):
    """HTML paragraph add form, JSON renderer"""


@pagelet_config(name='properties.html', context=IHTMLParagraph, layer=IPyAMSLayer,
                permission=MANAGE_CONTENT_PERMISSION)
class HTMLParagraphPropertiesEditForm(AdminDialogEditForm):
    """HTML paragraph properties edit form"""

    @property
    def title(self):
        content = get_parent(self.context, IWfSharedContent)
        return II18n(content).query_attribute('title', request=self.request)

    legend = _("Edit HTML paragraph properties")
    dialog_class = 'modal-max'
    icon_css_class = 'fa fa-fw fa-html5'
    label_css_class = 'control-label col-md-2'
    input_css_class = 'col-md-10'

    fields = field.Fields(IHTMLParagraph).omit('__parent__', '__name__', 'visible')
    ajax_handler = 'properties.json'
    edit_permission = MANAGE_CONTENT_PERMISSION

    def updateWidgets(self, prefix=None):
        super(HTMLParagraphPropertiesEditForm, self).updateWidgets(prefix)
        if 'body' in self.widgets:
            body_widget = self.widgets['body']
            for lang in body_widget.langs:
                widget = body_widget.widgets[lang]
                widget.id = '{id}_{name}'.format(id=widget.id, name=self.context.__name__)
            body_widget.widget_css_class = 'textarea'


class IHTMLParagraphInnerEditForm(Interface):
    """Marker interface for HTML paragraph inner form"""


@view_config(name='properties.json', context=IHTMLParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphPropertiesAJAXEditForm(BaseParagraphAJAXEditForm, HTMLParagraphPropertiesEditForm):
    """HTML paragraph properties edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HTMLParagraphPropertiesAJAXEditForm, self).get_ajax_output(changes)
        if 'body' in changes.get(IHTMLParagraph, ()):
            # refresh associations count markers
            parent = get_parent(self.context, IAssociationTarget)
            table = ParagraphContainerTable(parent, self.request)
            viewlet = ParagraphTitleToolbarViewletManager(parent, self.request, table)
            viewlet.update()
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.paragraphs.updateToolbar',
                            'object_name': self.context.__name__,
                            'toolbar_tag': viewlet.render()}})
            # refresh associations table
            associations_table = AssociationsTable(self.context, self.request)
            associations_table.update()
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.associations.refreshAssociations',
                            'object_name': associations_table.id,
                            'table': associations_table.render()}})
        return output


@adapter_config(context=(IHTMLParagraph, IPyAMSLayer), provides=IParagraphInnerEditor)
@implementer(IInnerForm, IPropertiesEditForm, IAssociationsParentForm, IHTMLParagraphInnerEditForm)
class HTMLParagraphInnerEditForm(HTMLParagraphPropertiesEditForm):
    """HTML paragraph inner edit form"""

    legend = None
    ajax_handler = 'inner-properties.json'

    @property
    def buttons(self):
        if self.mode == INPUT_MODE:
            return button.Buttons(IEditFormButtons)
        else:
            return button.Buttons()


@view_config(name='inner-properties.json', context=IHTMLParagraph, request_type=IPyAMSLayer,
             permission=MANAGE_CONTENT_PERMISSION, renderer='json', xhr=True)
class HTMLParagraphInnerAJAXEditForm(BaseParagraphAJAXEditForm, HTMLParagraphInnerEditForm):
    """HTML paragraph inner edit form, JSON renderer"""

    def get_ajax_output(self, changes):
        output = super(HTMLParagraphInnerAJAXEditForm, self).get_ajax_output(changes)
        if 'body' in changes.get(IHTMLParagraph, ()):
            # refresh associations count markers
            parent = get_parent(self.context, IAssociationTarget)
            table = ParagraphContainerTable(parent, self.request)
            viewlet = ParagraphTitleToolbarViewletManager(parent, self.request, table)
            viewlet.update()
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.paragraphs.updateToolbar',
                            'object_name': self.context.__name__,
                            'toolbar_tag': viewlet.render()}})
            # refresh associations table
            associations_table = AssociationsTable(self.context, self.request)
            associations_table.update()
            output.setdefault('events', []).append({
                'event': 'PyAMS_content.changed_item',
                'options': {'handler': 'PyAMS_content.associations.refreshAssociations',
                            'object_name': associations_table.id,
                            'table': associations_table.render()}
            })
        return output


#
# HTML paragraph summary
#

@adapter_config(context=(IHTMLParagraph, IPyAMSLayer), provides=IParagraphSummary)
@template_config(template='templates/html-summary.pt', layer=IPyAMSLayer)
class HTMLParagraphSummary(BaseContentProvider):
    """HTML paragraph summary"""

    illustration = None
    illustration_renderer = None
    language = None

    def update(self):
        i18n = II18n(self.context)
        if self.language:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.get_attribute(attr, self.language, request=self.request))
        else:
            for attr in ('title', 'body'):
                setattr(self, attr, i18n.query_attribute(attr, request=self.request))
        self.illustration = IIllustration(self.context)
        if self.illustration.data:
            registry = get_current_registry()
            renderer = self.illustration_renderer = registry.queryMultiAdapter((self.illustration, self.request),
                                                                               IIllustrationRenderer,
                                                                               name=self.illustration.renderer)
            if renderer is not None:
                renderer.update()

    def render_illustration(self):
        if not self.illustration_renderer:
            return ''
        return self.illustration_renderer.render()
